/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 5679 $
 */

/** @file
 * @brief Enhanced ShockBurst Primary Transmitter example
 * @defgroup esb_ptx_example Enhanced ShockBurst Primary Transmitter (PTX) example
 * @{
 * @ingroup nrf_examples
 * 
 * @brief This example sends packets continuously. The contents of P0 is 
 * sent in the first payload byte (byte 0). 
 *
 * The example shows the minimum required setup for transmitting packets to a 
 * primary receiver (PRX) device.  
 * 
 * The following default radio parameters are being used:
 * - RF channel 2
 * - 2 Mbps data rate
 * - TX address 0xE7E7E7E7E7 
 * - 1 byte CRC  
 * 
 * The project @ref esb_prx_example can be used as a counterpart for receiving the data.
 * 
*/

#include "nrf24le1.h"
#include <stdint.h>
#include <stdbool.h>
#include "hal_nrf.h"
#include "hal_nrf_hw.h"

// Global variables
bool radio_busy;

void main()

{
  uint8_t payload[3];
  // P1DIR as input for buttons
  P1DIR = 0xFF;
  // P0DIR as output for readback on leds
  P0DIR = 0x00;

  // Enable the radio clock
  RFCKEN = 1;

  // Enable RF interrupt
  RF = 1;  

  // Enable global interrupt
  EA = 1;
  
  // Enable dynamic payload on all pipes
  hal_nrf_setup_dynamic_payload(0xFF);
  //Enable Dynamic payload
  hal_nrf_enable_dynamic_payload(1);
  // Enable ACK with payload
  hal_nrf_enable_ack_payload(1);
  hal_nrf_enable_dynamic_ack(1);


  // Power up radio
  hal_nrf_set_power_mode(HAL_NRF_PWR_UP);


  while(1)
  {
    payload[1] = P1; // Button data 
    
    // Write payload to radio TX FIFO
    hal_nrf_write_tx_payload(payload, 3);

    // Toggle radio CE signal to start transmission 
    CE_PULSE();

    radio_busy = true;

    // Wait for radio operation to finish
    while(radio_busy)
    ;
  }                                           
} 

// Radio interrupt
NRF_ISR()
{
  uint8_t irq_flags;


  // Read and clear IRQ flags from radio
  irq_flags = hal_nrf_get_clear_irq_flags(); 

  switch(irq_flags)
  {
    // Transmission success
    case (1 << HAL_NRF_TX_DS):
      radio_busy = false;
      // Data has been sent
      break;
    // Transmission failed (maximum re-transmits)
    case (1 << HAL_NRF_MAX_RT):
      // When a MAX_RT interrupt occurs the TX payload will not be removed from the TX FIFO. 
      // If the packet is to be discarded this must be done manually by flushing the TX FIFO.
      // Alternatively, CE_PULSE() can be called re-starting transmission of the payload.
      // (Will only be possible after the radio irq flags are cleared) 
      hal_nrf_flush_tx();
      radio_busy = false;
      break;
    default:
    radio_busy = false;
    break;
  }
  if(!hal_nrf_rx_fifo_empty())
  {
    uint8_t rx_payload[3];
    // Read out rx_payload
    hal_nrf_read_rx_payload(rx_payload);
    // Output byte 1 on P0    
    P0 = ~rx_payload[1];
  }
}